# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
from aostools import *

class Create(Command):
    common = True
    helpSummary = "Create project or component from templet"
    helpUsage = """
%prog [project|comp]
"""
    helpDescription = """
Create project or component from templet
"""


    def Execute(self, opt, args):
        self.Usage()


# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

import os
from aostools import *


class Generate(Command):
    common = True
    helpSummary = "Generate manifest.yaml templet file"
    helpUsage = """
%prog
"""
    helpDescription = """
Generate manifest.yaml templet file
"""
    def _Options(self, p):
        p.add_option('-a', '--all',
                     dest='gen_all', action='store_true',
                     help='generate all components manifest.yaml')
        p.add_option('-s', '--solution',
                     dest='gen_solution', action='store_true',
                     help='generate current solution manifest.yaml')
        p.add_option('-k', '--sdk',
                     dest='gen_sdk', action='store', type='string',
                     help='generate sdk components manifest.yaml')
    def Execute(self, opt, args):
        yoc = YoC()
        mani = Manifest()
        if opt.gen_all:
            mani.gen_templet(yoc.components)
        elif opt.gen_solution:
            solution = yoc.getSolution()
            if solution == None:
                put_string("The current directory is not a solution!")
                return 1
            mani.gen_templet(solution.components)
        elif opt.gen_sdk:
            print(opt.gen_sdk)
            components = ComponentGroup()
            sdk_cmpt = yoc.components.get(opt.gen_sdk)
            if sdk_cmpt:
                sdk_cmpt.load_package()
                for name in sdk_cmpt.depends:
                    if type(name) == dict:
                        name = list(name.keys())[0]
                    cmpt = yoc.components.get(name)
                    if cmpt:
                        components.add(cmpt)
                mani.gen_templet(components)
            else:
                put_string("The sdk component %s is not exist." % opt.gen_sdk)
                return 1
        return 0



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


import os
import sys
import pickle

from .tools import *
from .log import logger
from .yoc import YoC
from .builder import *
from .package import set_specified_board

import warnings
warnings.filterwarnings("ignore", category=SyntaxWarning)

try:
    try:
        import SCons.Script as SCons
    except:
        import scons
        for path in scons.__path__:
            sys.path.append(path)
            import SCons.Script as SCons
except:
    scons_path = []
    for path in sys.path:
        if os.path.isdir(os.path.join(path, "scons")):
            scons_path.append(os.path.join(path, "scons"))
    if scons_path:
        sys.path += scons_path
        import SCons.Script as SCons


SCons.AddOption('--verbose', dest='verbose', default=True,
                action='store_false',
                help='verbose command line output')


SCons.AddOption('--board', dest='board', default=None,
                action='store', nargs=1, type='string',
                help='board component name')

class Make(object):
    def __init__(self, elf=None, objcopy=None, objdump=None):
        try:
            with open('.sconsign.dblite', "rb") as f:
                pickle.load(f)
        except ValueError:
            os.remove('.sconsign.dblite')
        except:
            pass

        board_name = SCons.GetOption('board')
        set_specified_board(board_name)
        print_depends = False
        if not SCons.GetOption('verbose'):
            print_depends = True
        self.yoc = YoC()
        solution = self.yoc.getSolution(board_name, print_depends=print_depends, exit_if_lost=False)
        if not solution:
            # retry if auto install missing component
            self.yoc = YoC()
            solution = self.yoc.getSolution(board_name, print_depends=print_depends)
        if solution:
            if not SCons.GetOption('verbose'):
                put_string("----------------------------------------------------------------")
                for comp in solution.cond_components:
                    if comp.conditions:
                        put_string(comp.name + ":", comp.conditions)
                put_string("----------------------------------------------------------------")
            filename = "%s@%s" % (solution.solution_component.name, solution.solution_component.hw_info.board_name)
            if not elf:
                elf = os.path.join("out", "%s.elf" % filename)
            if not objcopy:
                objcopy = os.path.join("out", "%s.bin" % filename)
            self.build_env = Builder(solution)
            self.build_env.env.AppendUnique(
                ELF_FILE=elf, OBJCOPY_FILE=objcopy, OBJDUMP_FILE=objdump)
            # self.build_env.env.AppendUnique(BOARD_NAME=board, CHIP_NAME=chip)
        else:
            self.build_env = None

    def library_yaml(self):
        package_file = self.build_env.env.GetBuildPath('package.yaml')
        conf = yaml_load(package_file)
        if conf and 'name' in conf:
            component = self.yoc.components.get(conf['name'])
            self.build_env.build_component(component)
            component.export()
            if component.name in ['separate_app']:
                os.remove(package_file)

    def build_component(self, component):
        new_file = False
        file_name = os.path.join(component.path, 'SConscript')
        if not os.path.exists(file_name):
            file_name = genSConcript(component.path)
            new_file = True
        if os.path.isfile(file_name):
            SCons.SConscript(file_name, exports={"env": self.build_env.env.Clone(
            )}, variant_dir='out/' + component.name, duplicate=0)
            if new_file:
                os.remove(file_name)

    def build_components(self, components=None):
        if self.build_env.env['PLATFORM'] == 'win32':
            self.build_env.env['SPAWN'] = win_longcmd_spawn
            put_string("install win_longcmd_spawn patch in windows.")
        if components != None:
            for c in components:
                component = self.yoc.components.get(c)
                if component:
                    self.build_component(component)
        else:
            for component in self.build_env.solution.components:
                self.build_component(component)

    def build_image(self, elf=None, objcopy=None, objdump=None, product=None):
        if self.build_env.env['PLATFORM'] == 'win32':
            self.build_env.env['SPAWN'] = win_longcmd_spawn
            put_string("install win_longcmd_spawn patch in windows.")
        self.build_env.build_image(elf, objcopy, objdump, product)

def win_longcmd_spawn(sh, escape, cmd, args, spawnenv):
    import win32file
    import win32event
    import win32process
    import win32security
    from SCons.Platform.win32 import spawn

    if cmd.startswith('arm-') or cmd.startswith('csky-') or cmd.startswith('riscv-'):
        #put_string("use win_longcmd_spawn patch for %s in windows." % cmd)
        newargs = ' '.join(args[1:])
        for p in spawnenv["PATH"].split(";"):
            exe_file = os.path.join(p, "%s.exe" % cmd)
            if os.path.isfile(exe_file):
                cmd = exe_file
                break
        cmdline = cmd + " " + newargs
        handle = win32process.CreateProcess(None, cmdline, None, None, 1, 0, spawnenv, None, win32process.STARTUPINFO())
        win32event.WaitForSingleObject(handle[0], win32event.INFINITE)
        exit_code = win32process.GetExitCodeProcess(handle[0])
        win32file.CloseHandle(handle[0])
        win32file.CloseHandle(handle[1])
    else:
        exit_code = spawn(sh, escape, cmd, args, spawnenv)
    return exit_code



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
from aostools import *

import codecs


class Cdk(Command):
    common = True
    helpSummary = "Generate current solution SDK"
    helpUsage = """
%prog
"""
    helpDescription = """
generate current solution SDK.
"""
    # def _Options(self, p):
    #     p.add_option('-a', '--all',
    #                  dest='show_all', action='store_true',
    #                  help='show the complete list of commands')

    def Execute(self, opt, args):
        yoc = YoC()
        solution = yoc.getSolution()
        if solution:
            packages=''
            for c in solution.components:
                packages += '    <Package ID="%s" Version="%s" IsBasic="false"/>\n' % (c.name, c.version)

            text = temp.format(
                name=solution.solution_component.name,
                description=solution.solution_component.description,
                chip_name=solution.chip_component.name,
                chip_version=solution.chip_component.version,
                board_name=solution.board_component.name,
                board_version=solution.board_component.version,
                packages=packages,
                cpu_name=solution.cpu_name,
                CFLAGS=' '.join(solution.CFLAGS),
                LINKFLAGS=' '.join(solution.LINKFLAGS),
                ld_script=solution.ld_script,
            )

            generate_flash_init(os.path.join(solution.board_component.path, 'configs/config.yaml'),
                os.path.join(solution.solution_component.path, 'script/flash.init')
            )
            try:
                shutil.copytree('/usr/local/lib/aostools/script', os.path.join(solution.solution_component.path, 'script'))
            except:
                pass

            try:
                filename = os.path.join(solution.solution_component.path, 'project.cdkproj')
                with codecs.open(filename, 'w', 'UTF-8') as f:
                    f.write(text)
                return True
            except:
                put_string("Generate %s file failed." % filename)



temp='''
<?xml version="1.0" encoding="UTF-8"?>
<Project Name="{name}" Version="1">
  <Description>{description}</Description>
  <MonitorProgress>
    <FlashOperate>230</FlashOperate>
    <DebugLaunch>42</DebugLaunch>
  </MonitorProgress>
  <Chips>
    <Chip ID="{chip_name}" Version="{chip_version}" IsBasic="false"/>
  </Chips>
  <Boards>
    <Board ID="{board_name}" Version="{board_version}" IsBasic="false"/>
  </Boards>
  <Packages>
{packages}  </Packages>
  <MergedToYaml>yes</MergedToYaml>
  <BuildConfigs>
    <BuildConfig Name="BuildSet">
      <Target>
        <ROMBank Selected="1">
          <ROM1>
            <InUse>yes</InUse>
            <Start>0x20000000</Start>
            <Size>0x1000</Size>
          </ROM1>
          <ROM2>
            <InUse>no</InUse>
            <Start/>
            <Size/>
          </ROM2>
          <ROM3>
            <InUse>no</InUse>
            <Start/>
            <Size/>
          </ROM3>
          <ROM4>
            <InUse>no</InUse>
            <Start/>
            <Size/>
          </ROM4>
          <ROM5>
            <InUse>no</InUse>
            <Start/>
            <Size/>
          </ROM5>
        </ROMBank>
        <RAMBank>
          <RAM1>
            <InUse>no</InUse>
            <Start>0x20001000</Start>
            <Size>0x1000</Size>
            <Init>yes</Init>
          </RAM1>
          <RAM2>
            <InUse>no</InUse>
            <Start/>
            <Size/>
            <Init>yes</Init>
          </RAM2>
          <RAM3>
            <InUse>no</InUse>
            <Start/>
            <Size/>
            <Init>yes</Init>
          </RAM3>
          <RAM4>
            <InUse>no</InUse>
            <Start/>
            <Size/>
            <Init>yes</Init>
          </RAM4>
          <RAM5>
            <InUse>no</InUse>
            <Start/>
            <Size/>
            <Init>yes</Init>
          </RAM5>
        </RAMBank>
        <CPU>{cpu_name}</CPU>
        <UseMiniLib>yes</UseMiniLib>
        <Endian>little</Endian>
        <UseHardFloat>no</UseHardFloat>
        <UseEnhancedLRW>no</UseEnhancedLRW>
        <UseContinueBuild>no</UseContinueBuild>
        <UseSemiHost>no</UseSemiHost>
      </Target>
      <Output>
        <OutputName>$(ProjectName)</OutputName>
        <Type>Executable</Type>
        <CreateHexFile>yes</CreateHexFile>
        <CreateBinFile>no</CreateBinFile>
        <Preprocessor>no</Preprocessor>
        <Disasm>no</Disasm>
        <CallGraph>no</CallGraph>
        <Map>no</Map>
      </Output>
      <User>
        <BeforeCompile>
          <RunUserProg>no</RunUserProg>
          <UserProgName/>
        </BeforeCompile>
        <BeforeMake>
          <RunUserProg>yes</RunUserProg>
          <UserProgName>"$(ProjectPath)/script/pre_build.sh"</UserProgName>
        </BeforeMake>
        <AfterMake>
          <RunUserProg>yes</RunUserProg>
          <UserProgName>"$(ProjectPath)/script/aft_build.sh"</UserProgName>
        </AfterMake>
      </User>
      <Compiler>
        <Define/>
        <Undefine/>
        <Optim>Optimize size (-Os)</Optim>
        <DebugLevel>Default (-g)</DebugLevel>
        <IncludePath>$(ProjectPath)/src/tftp;$(ProjectPath)cb5654/include;$(ProjectPath)include;$(ProjectPath)src/audio;$(ProjectPath)src/wifipair;$(ProjectPath)src/wifipair/ap</IncludePath>
        <OtherFlags>{CFLAGS}</OtherFlags>
        <Verbose>no</Verbose>
        <Ansi>no</Ansi>
        <Syntax>no</Syntax>
        <Pedantic>no</Pedantic>
        <PedanticErr>no</PedanticErr>
        <InhibitWarn>no</InhibitWarn>
        <AllWarn>yes</AllWarn>
        <WarnErr>no</WarnErr>
        <OneElfS>no</OneElfS>
        <Fstrict>no</Fstrict>
      </Compiler>
      <Asm>
        <Define/>
        <Undefine/>
        <IncludePath>$(ComProjectPath)include;$(ProjectPath)/src/tftp;$(ProjectPath)src;$(ProjectPath)src/audio;$(ProjectPath)src/wifipair;$(ProjectPath)src/wifipair/ap</IncludePath>
        <OtherFlags/>
        <DebugLevel>gdwarf2</DebugLevel>
      </Asm>
      <Linker>
        <Garbage>yes</Garbage>
        <LDFile>$(ProjectPath){ld_script}</LDFile>
        <LibName/>
        <LibPath/>
        <OtherFlags>{LINKFLAGS}</OtherFlags>
        <AutoLDFile>no</AutoLDFile>
      </Linker>
      <Debug>
        <LoadApplicationAtStartup>no</LoadApplicationAtStartup>
        <Connector>ICE</Connector>
        <StopAt>no</StopAt>
        <StopAtText>main</StopAtText>
        <InitFile>$(ProjectPath)script/cdkgdbinit</InitFile>
        <AutoRun>no</AutoRun>
        <ResetType>Hard Reset</ResetType>
        <SoftResetVal/>
        <ResetAfterLoad>no</ResetAfterLoad>
        <Dumpcore>no</Dumpcore>
        <DumpcoreText>$(ProjectPath)$(ProjectName).cdkcore</DumpcoreText>
        <ConfigICE>
          <IP>localhost</IP>
          <PORT>1025</PORT>
          <Clock>12000</Clock>
          <Delay>10</Delay>
          <WaitReset>500</WaitReset>
          <DDC>yes</DDC>
          <TRST>no</TRST>
          <DebugPrint>no</DebugPrint>
          <Connect>Normal</Connect>
          <ResetType>Soft Reset</ResetType>
          <SoftResetVal>0</SoftResetVal>
          <RTOSType>Bare Metal</RTOSType>
          <DownloadToFlash>yes</DownloadToFlash>
          <ResetAfterConnect>no</ResetAfterConnect>
          <GDBName/>
          <GDBServerType>Local</GDBServerType>
          <OtherFlags/>
        </ConfigICE>
        <ConfigSIM>
          <SIMTarget>soccfg/cskyv2/rhea802.xml</SIMTarget>
          <OtherFlags/>
          <NoGraphic>yes</NoGraphic>
          <Log>no</Log>
          <SimTrace>no</SimTrace>
        </ConfigSIM>
      </Debug>
      <Flash>
        <InitFile>$(ProjectPath)script/flash.init</InitFile>
        <Erase>Erase Sectors</Erase>
        <Algorithms Path="">SC5654A_SPIFlash_CDK.elf</Algorithms>
        <Program>yes</Program>
        <Verify>yes</Verify>
        <ResetAndRun>no</ResetAndRun>
        <ResetType>Soft Reset</ResetType>
        <SoftResetVal/>
        <External>no</External>
        <Command/>
        <Arguments/>
      </Flash>
    </BuildConfig>
  </BuildConfigs>
  <DebugSessions>
    <watchExpressions/>
    <memoryExpressions/>
    <statistics/>
    <peripheralTabs/>
    <WatchDisplayFormat/>
    <LocalDisplayFormat/>
    <debugLayout/>
    <memoryTabColSizeExpressions/>
  </DebugSessions>
</Project>
'''


def generate_flash_init(config_yaml, filename):
    text = '''
#reset
set $psr=0xc0000000
set *(int*)0x42010300=0x03

{downloads} '''
    v = yaml_load(config_yaml)
    cmd = 'download ihex verify=no $(ProjectPath)generated/{name}.hex'
    downloads=[]
    for k in v['partitions']:
        # put_string(k)
        f = cmd.format(name=k['name'])
        downloads.append(f)

    downloads = '\n'.join(downloads)
    # put_string(s)
    text = text.format(downloads=downloads)
    try:
        with codecs.open(filename, 'w', 'UTF-8') as f:
            f.write(text)
        return True
    except:
        put_string("Generate %s file failed." % filename)


# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

import threadpool, re
from aostools import *


class Install(Command):
    common = True
    helpSummary = "Install component into project environment"
    helpUsage = """
%prog [option] [<component>...]
"""
    helpDescription = """
Install component into project environment
"""

    def _Options(self, p):
        self.jobs = 1
        p.add_option('-j', '--jobs',
                     dest='jobs', action='store', type='int',
                     help="projects to fetch simultaneously (default %d)" % self.jobs)
        p.add_option('-f', '--force',
                     dest='force', action='store_true',
                     help='install component force if exist already')
        p.add_option('-b', '--branch',
                     dest='branch', action='store', type='string',
                     help='the branch for component to download')
        p.add_option('-L', '--local',
                     dest='local', action='store_true',
                     help='install component from local zip file')

    def Execute(self, opt, args):
        if opt.local:
            return self.local_comp(opt, args)
        else:
            return self.remote_comp(opt, args)

    def local_comp(self, opt, args):
        yoc = YoC()
        if len(args) <= 0:
            put_string("No component file provided!")
            return 1
        
        pattern = re.compile(r'(.*)-(.*)\.zip')
        for zipfile in args:
            if not os.path.exists(zipfile):
                put_string("component file \"%s\" is not exist!" % zipfile)
                continue
            match = pattern.match(os.path.basename(zipfile))
            if match:
                name = match.group(1)
                version = match.group(2)
                component = yoc.components.get(name)
                if component:
                    if opt.force:
                        if unzip_path(yoc.yoc_path, zipfile):
                            put_string("install component \"%s\" successful" % name)
                    else:
                        put_string("Component `%s` have installed already! Please add -f option to install force!" % name)
                else:
                    if unzip_path(yoc.yoc_path, zipfile):
                        put_string("install component \"%s\" successful" % name)
            else:
                put_string("invalid component zip file \"%s\", it should be <name>-<version>.zip" % zipfile)
        return 0


    def remote_comp(self, opt, args):
        if opt.jobs:
            jobs = opt.jobs
        else:
            jobs = 12
        put_string("Start to install components...")
        yoc = YoC()
        components = ComponentGroup()
        if len(args) > 0:
            for name in args:
                update = False
                if name == args[0]:
                    update = True
                cmpt = yoc.check_cmpt_download(name, update=True, force=opt.force)
                if cmpt:
                    components.add(cmpt)
        else:
            yoc.update()
            components = yoc.occ_components
        
        if len(components) > 0:
            dled_cmpts =[]
            while len(components) > 0:
                cmpts = components
                self.download(jobs, cmpts, opt.branch)
                for c in cmpts:
                    if c.name not in dled_cmpts:
                        dled_cmpts.append(c.name)
                components = ComponentGroup()
                for c in cmpts:
                    ret = c.load_package()
                    if ret:
                        yoc.update_version(c.depends)
                        cmpt_list = self.get_need_download_cmpts(args, dled_cmpts, c.depends)
                        validComponentList = []
                        for component in yoc.occ_components:
                            if component.name in cmpt_list:
                                components.add(component)
                                validComponentList.append(component.name)
                        invalidComponentList = set(cmpt_list).difference(set(validComponentList))
                        if invalidComponentList:
                            put_string("warning: These components [%s] are not existed!" % (", ").join(invalidComponentList))

            put_string('Download components finish.')
            return 0
        else:
            put_string("No component need to install.")
            return 1

    def get_need_download_cmpts(self, origin_list, downloaded_list, depends):
        cmpt_list = []
        for name in depends:
            if type(name) == dict:
                name = list(name.keys())[0]
                if (name not in origin_list) and (name not in downloaded_list):
                    cmpt_list.append(name)
        return cmpt_list

    def download(self, jobs, components, branch):
        task_pool = threadpool.ThreadPool(jobs)
        tasks = []
        for component in components:
            tasks.append(component)

        def thread_execture(component):
            component.download(branch)

        requests = threadpool.makeRequests(thread_execture, tasks)
        for req in requests:
            task_pool.putRequest(req)
        task_pool.wait()
        task_pool.dismissWorkers(jobs, do_join=True)
